import { SSE } from 'sse.js';
import type { StreamMessage } from './models';
import { DateTime } from 'luxon';

const API_SUFFIX = "/api/v1";

export const urlFor = (path: string) => {
    return window.location.protocol + "//" + window.location.hostname + API_SUFFIX + "/" + path;
};

export const loadData = async (year: number, day: number, mode: "holt59" | "tests") => {
    const url = new URL(urlFor("data"));
    url.searchParams.append("year", year.toString());
    url.searchParams.append("day", day.toString());
    url.searchParams.append("mode", mode);
    return await fetch(url, {
        "method": "GET",
    }).then(r => r.json()).then(r => r["content"]).catch(() => "");
};

export const submit = async (year: number, day: number, input: string, verbose: boolean, startCallback: (time: DateTime, commit: string) => void, streamCallback: (message: StreamMessage) => void) => {
    const data = new FormData();
    data.append("year", year.toString());
    data.append("day", day.toString());
    data.append("input", input.toString());
    data.append("verbose", verbose.toString());

    return new Promise((resolve) => {
        const eventSource = new SSE(urlFor("submit-sse"), {
            "start": false,
            // @ts-expect-error bad typing from SSE
            "payload": data
        });

        eventSource.addEventListener('message', function (e: { data: string }) {
            try {
                const data = JSON.parse(e.data);

                if (data.event == "complete") {
                    resolve(null)
                }
                else if (data.event == "start") {
                    startCallback(DateTime.fromISO(data.data.time), data.data.commit);
                }
                else if (data.event == "stream") {
                    streamCallback(data.data);
                }
            }
            catch (exception) {
                console.log(e.data);
                console.log(exception);
            }
        });

        eventSource.stream();
    });
}
